#include <avr/io.h>               

//Definition von Taktfrequenz und Bausrate als Konstanten
#define TAKTFREQUENZ 1000000
#define BAUDRATE 2400


void init()
{
   DDRA = 0;
   DDRB = 0;
   DDRC = 0;
   DDRD = 0;

   DDRD |= 0b00110010;
   PORTD &= 0b11111101; //PD1==TXD hier einfach auf Masse ziehen!

//Grne LED Grn1 vorbereiten:
   DDRA |= 0b00000011;  //PA0 und PA1 als Ausgang konfigurieren          
   PORTA &= 0b11111100; //PA0 und PA1 auf Masse ziehen.      
//Rote LED Rot1 vorbereiten:
   DDRB  |= 0b00011000;
   PORTB &= 0b11100111;


//Gegenpol der IR-Dioden mit Trgerfrequenz von 30kHz konfigurieren:
   ICR1 = 17;  //fr 1MHz

   TCCR1A = (1<<COM1A1) | (0<<COM1A0) | (1<<COM1B1) | (0<<COM1B0) 
          | (0<<FOC1A) | (0<<FOC1B) | (1<<WGM11) | (0<<WGM10);
   TCCR1B = (0<<ICNC1) | (0<<ICES1) | (1<<WGM13) | (0<<WGM12) 
          | (0<<CS12) | (0<<CS11) | (1<<CS10);

   OCR1A = 8; //PWM-Breite auf 50% setzen.
   OCR1B = 8; //PWM-Breite auf 50% setzen.


//serielle Schnittstelle aktivieren:
   //Merken des in UBRR zu speichernden Wertes.
   //(TAKTFREQUENZ/8/BAUDRATE-1); //1MHz: 1000000/8/2400 - 1 = 51,08 (o.k., kleiner Fehler)

   //setzen der Baudrate
   UBRRH = (unsigned char) ((TAKTFREQUENZ/8/BAUDRATE-1)>>8); //Setzen des HIGH-Bytes des Baudraten-Registers
   UBRRL = (unsigned char)  (TAKTFREQUENZ/8/BAUDRATE-1);     //Setzen des LOW -Bytes des Baudraten-Registers
	
   //Einschalten des Senders und des Empfngers
   //UCSRB = (1<<TXEN) | (1<<RXEN);
		        
   //Setzen des Nachrichtenformats: 8 Datenbits, 1 Stopbits
   UCSRC = (1<<URSEL)|(1<<UCSZ0)|(1<<UCSZ1);
   UCSRA = (1<<U2X);
}

int main(void)                    
{   
   unsigned char akku = ' ';
   int zaehler = 0;
   int timeout = 0;

   unsigned char es_ist_jemand_in_der_naehe = 0;
   unsigned char ich_bin_bemerkt_worden     = 0;

   init();

   while(1)                       
   {        
        //LEDs gem Zustand schalten:
        //nur rot: niemand in der Nhe:
        if( es_ist_jemand_in_der_naehe==0 )
        {
            //nur rot
            PORTB |= 0b00010000; //PB4 auf High, Rot1 einschalten.
            PORTA &= 0b11111101; //PA1 sicher auf 0 schalten. Grn1 ausschalten
        }
        else if( ich_bin_bemerkt_worden==0 ) //da ist jemand, aber dieser hat micht nicht bemerkt.
        {
            //rot und grn
            PORTB |= 0b00010000; //PB4 auf High, Rot1 einschalten.
            PORTA |= 0b00000010; //PA1 auf High schalten. Grn1 einschalten
        }
        else
        {
            //nur grn: da ist jemand, und ich wurde bemerkt.
            PORTB &= 0b11101111; //PB4 auf Low, Rot1 ausschalten.
            PORTA |= 0b00000010; //PA1 auf High schalten. Grn1 einschalten
        }

        if(zaehler<80) //versuche sehr oft etwas zu empfangen
        {
            timeout = 100;
            while( !(UCSRA & (1<<RXC)) && (timeout--) > 0);

            akku = UDR;                    //Zeichen aus dem Puffer holen

                
            if(akku=='A')
            {
                es_ist_jemand_in_der_naehe = 1;
                ich_bin_bemerkt_worden = 0;
            }
            else if(akku=='B')
            {
                es_ist_jemand_in_der_naehe = 1;
                ich_bin_bemerkt_worden = 1;
            }
        }

        if(zaehler>=80) //sende manchmal etwas
        {
            if(es_ist_jemand_in_der_naehe==1)
            {
                while( !(UCSRA & (1<<UDRE)) ); //Warten bis der Uebertragungspuffer leer ist
                UDR = 'B';                    //Daten in den Puffer schreiben und bertragen
            }
            else
            {
                while( !(UCSRA & (1<<UDRE)) ); //Warten bis der Uebertragungspuffer leer ist
                UDR = 'A';                    //Daten in den Puffer schreiben und bertragen
            }
        }                

        zaehler++;
        zaehler%=100;

        if(zaehler==0)
        {
            akku = ' '; //zurcksetzen
        }
   }                              
}

