package kugelsim;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.ArrayList;

import  javax.sound.midi.MidiSystem;
import  javax.sound.midi.Synthesizer;
import  javax.sound.midi.MidiChannel;
import  javax.sound.midi.MidiUnavailableException;


public class Kugelsim extends Applet
{
    private Synthesizer	synth = null;
    private MidiChannel[] channels;
    private int[] miditon;
    private int[] centton;
    private int[] pitchton;

    private int[] tonlautstaerke;
    private int[] tonposition;

    private boolean[] tonan;
    private int[] periode;
    private int[] periode_neu;

    /**Durch Multiplikation einer Frequenz mit diesem Faktor, 
    wird sie einen halben Ton hher.*/
    private double halbton_faktor;

    /**Hiermit muss man den Ton 100mal multiplizieren, 
    um ihn einen halben Ton hher zu machen.*/  
    private double eincent_faktor;  
    private double pitch_faktor;  
    private double frequenz_tiefster_ton;



    private long zeit      = 0;
    private long ZEIT      = 0;
    private long ZEITALT   = 0;
    private long ZEITDELTA = 50;

    private Kugelsim kugelsim;

    private Point mauspos = null;

    private int distanzneu,distanzalt;

    private int xalt = 0;
    private int yalt = 0;
    private int xneu = 0;
    private int yneu = 0;

    private int XOFF = 20;
    private int YOFF = 20;
    private int RADIUS = 30;
    private int KASTEN = 200;
    private int BREITE = 800;
    private int HOEHE  = 600;

    private int anzkug = 8;

    private ArrayList<Kugel> kugeln;

    private boolean kollision(int x1, int y1, int x2, int y2)
    {
        if( (x1-x2)*(x1-x2)+(y1-y2)*(y1-y2) < 16*RADIUS*RADIUS )
            return true;
        else
            return false;
    }

    private void neueFrequenz()
    {
        double freq = 220.0;
        double fbasis = 220.0;

        for(int i=0;i<anzkug;i++)
        {
            if(periode_neu[i]<=8)
            {
                freq = fbasis;

                for(int k=8-periode_neu[i];k>0;k--)
                    freq+=fbasis;                
            }
            else if(periode_neu[i]<=16)
            {
                freq = fbasis*3.0/2.0;

                for(int k=8-(periode_neu[i]-8);k>0;k--)
                    freq+=fbasis;                
            }
            else if(periode_neu[i]<=24)
            {
                freq = fbasis*4.0/3.0;

                for(int k=8-(periode_neu[i]-16);k>0;k--)
                    freq+=fbasis;                
            }

            miditon[i] = bestimmeMiditon(freq);
            centton[i] = bestimmeCent(freq);
            pitchton[i] = bestimmePitch(freq);

            channels[i].setPitchBend(pitchton[i]);

            freq+=fbasis;
        }
    }

    private void findePeriodeNeu()
    {
        for(int i=0;i<anzkug;i++)
            periode_neu[i] = periode[i];

        for(int i=0;i<anzkug;i++)
        {
            for(int k=0;k<anzkug;k++)
            {
                if(i!=k && kollision(kugeln.get(i).x,kugeln.get(i).y,kugeln.get(k).x,kugeln.get(k).y))
                    periode_neu[i]+=periode[k];
            }
        }       

        for(int i=0;i<anzkug;i++)
            kugeln.get(i).farbe = new Color(getRot(periode_neu[i]-1),getGruen(periode_neu[i]-1),getBlau(periode_neu[i]-1));
    }

    private void setzeLautPos(int nr)
    {
        tonlautstaerke[nr] = ((HOEHE - kugeln.get(nr).y)*60)/HOEHE + 7;
//        tonposition[nr]   = ((kugeln.get(nr).x-BREITE/2)*63)/(BREITE/2);
        tonposition[nr]   = (kugeln.get(nr).x*127)/BREITE;
        channels[nr].controlChange(10, tonposition[nr]); //Pan
    }

    private int bestimmeMiditon(double frequenz)
    {
        int    midi = 0;
        double freq = frequenz_tiefster_ton;            

        while(freq<frequenz)
        {
            midi++;
            freq*=halbton_faktor;
        }

        if( (freq-frequenz) > (frequenz-freq/halbton_faktor) )
        {
            midi--;
            freq/=halbton_faktor;
        }

        return midi;
    }

    private int bestimmeCent(double frequenz)
    {
        int    midi = 0;
        double freq = frequenz_tiefster_ton;            
        int    cent = 0;
 
        while(freq<frequenz)
        {
            midi++;
            freq*=halbton_faktor;
        }

        if( (freq-frequenz) > (frequenz-freq/halbton_faktor) )
        {
            midi--;
            freq/=halbton_faktor;

            while(freq<frequenz)
            {
                freq*=eincent_faktor;
                cent++;
            }
        }
        else
        {
            while(freq>frequenz)
            {
                freq/=eincent_faktor;
                cent--;
            }
        }

        return cent;
    }

    private int bestimmePitch(double frequenz)
    {
        int    midi = 0;
        double freq = frequenz_tiefster_ton;            
        int    pitch = 8192;
 
        while(freq<frequenz)
        {
            midi++;
            freq*=halbton_faktor;
        }

        if( (freq-frequenz) > (frequenz-freq/halbton_faktor) )
        {
            midi--;
            freq/=halbton_faktor;

            while(freq<frequenz)
            {
                freq*=pitch_faktor;
                pitch++;
            }
        }
        else
        {
            while(freq>frequenz)
            {
                freq/=pitch_faktor;
                pitch--;
            }
        }

        return pitch;
    }

    private void interrupt()
    {
//        System.out.println("Interrupt zeit = "+zeit);
String ein = "";
        for(int i=0;i<anzkug;i++)
        {
            if(zeit%periode_neu[i]==0)
            {
                if(tonan[i]==false)
                {
ein+=i+" ";
                    channels[i].noteOn(miditon[i],tonlautstaerke[i]); 
                    tonan[i] = true;
                }
                else
                {
                    channels[i].noteOff(miditon[i]); 
                    tonan[i] = false;
                }
            }
        }
System.out.println("ein = "+ein);
        zeit++;
    }

    private void aktiviereTon()
    {
        findePeriodeNeu();
        neueFrequenz();
        for(int i=0;i<channels.length;i++)
            channels[i].allNotesOff();
        for(int i=0;i<tonan.length;i++)
            tonan[i] = false;

        System.out.println("Ton aktiviert");
    }

    private void deaktiviereTon()
    {
        for(int i=0;i<channels.length;i++)
            channels[i].allNotesOff();
        for(int i=0;i<tonan.length;i++)
            tonan[i] = false;

        System.out.println("Ton deaktiviert");
    }

    private int getRot(int nr)
    {
        return 200 - 50*(nr%4);        
    }

    private int getGruen(int nr)
    {
        return 200 - 50*((nr/4)%4);        
    }

    private int getBlau(int nr)
    {
        return 200 - 50*((nr/16)%4);        
    }

    public void init()
    {
        kugelsim = this;

        try
        {
            synth = MidiSystem.getSynthesizer();
            synth.open();
            channels = synth.getChannels(); 
            for(int i=0;i<anzkug;i++)
                channels[i].programChange(0, 11); //11: Marimba

        }
        catch (MidiUnavailableException e)
        {
        }

        miditon = new int[anzkug];
        centton = new int[anzkug];
        pitchton = new int[anzkug];

        tonlautstaerke = new int[anzkug];
        tonposition    = new int[anzkug];


        periode  = new int[anzkug];
        periode_neu  = new int[anzkug];

        tonan    = new boolean[anzkug];

        for(int i=0;i<tonan.length;i++)
            tonan[i] = false;

        halbton_faktor = Math.pow(2.0,(1.0/12.0));
        eincent_faktor = Math.pow(2.0,(1.0/1200.0));
        pitch_faktor = Math.pow(2.0,(1.0/(12.0*4096.0)));

        //Midi  Bez.  Freq./Hz
        //69 == a'  == 440Hz
        //57 == a   == 220Hz
        //45 == A   == 110Hz
        //33 == KA  == 55Hz
        //21 == SKA == 27.5Hz
        // 9 ==     == 13.75Hz
        // 0 ==        13.75Hz / (2^(1/12))^9 
        frequenz_tiefster_ton = 13.75/Math.pow(halbton_faktor,9);

        double freq = 220.0;
        double fbasis = 220.0;

        int perioden_basis = 840; //1*5*3*7*8 (kgv fr 1,2,3,4,5,6,7,8)

        for(int i=0;i<anzkug;i++)
        {
            miditon[i] = bestimmeMiditon(freq);
            centton[i] = bestimmeCent(freq);
            pitchton[i] = bestimmePitch(freq);

            channels[i].setPitchBend(pitchton[i]);

//            periode[i] = perioden_basis/(i+1);
            periode[i] = 8-i;
            periode_neu[i] = 8-i;

            freq+=fbasis;
        }
    }

    public void start()
    {
        this.setBackground(Color.WHITE);

        kugeln = new ArrayList<Kugel>();
        for(int i=0;i<anzkug;i++)
        {
            kugeln.add(new Kugel(i));
        }

        for(int i=0;i<anzkug;i++)
            setzeLautPos(i);

        this.addMouseListener(new MausLauscher());
    }

    public void paint(Graphics gg)
    {
        Graphics2D g = (Graphics2D)gg;

        mauspos = getMousePosition();

        if(kugeln!=null && kugeln.size()>0)
        {
            for(int i=0;i<kugeln.size();i++)
               kugeln.get(i).zeichnen(g);
        }


        while(mauspos!=null)
        {    
            ZEIT = System.currentTimeMillis();
            if(ZEIT>ZEITALT+ZEITDELTA)
            {
                ZEITALT = ZEIT;
                interrupt();         
            }

            xalt = xneu;
            yalt = yneu;

            xneu = (int)mauspos.getX();
            yneu = (int)mauspos.getY();   

            if(xneu!=xalt || yneu!=yalt)
            {   
                for(int i=0;i<kugeln.size();i++)
                {                    
                    distanzneu = kugeln.get(i).distanz(xneu,yneu);
                    distanzalt = kugeln.get(i).distanz(xalt,yalt);
                    if(distanzneu<0 && distanzalt<0 && distanzneu<distanzalt)
                    {
                        kugeln.get(i).x += xneu-xalt;
                        kugeln.get(i).y += yneu-yalt;
                        setzeLautPos(i);
                        findePeriodeNeu();
                        neueFrequenz();
                    }
                }

                if(kugeln!=null && kugeln.size()>0)
                {
                    for(int i=0;i<kugeln.size();i++)
                    {
                        kugeln.get(i).loeschen(g);
                        kugeln.get(i).zeichnen(g);
                    }
                }
            }
            mauspos = getMousePosition();
            try
            {
                Thread.sleep(5);
            }
            catch(Exception e)
            {
            }
        }

        deaktiviereTon();
    }

    private class Kugel
    {
        public int nr;
        public int x;
        public int y;

        public int x_alt;
        public int y_alt;

        private Color farbe;

        public int distanz(int xx, int yy) //<0, wenn drin
        {
            return (xx-x-RADIUS)*(xx-x-RADIUS)+(yy-y-RADIUS)*(yy-y-RADIUS) - RADIUS*RADIUS; 
        }

        public Kugel(int nr_in)
        {
            nr = nr_in;

            farbe = new Color(getRot(nr),getGruen(nr),getBlau(nr));


            x = KASTEN*(nr%(BREITE/KASTEN)) + KASTEN/2 - RADIUS + XOFF;
            y = KASTEN*(nr/(BREITE/KASTEN)) + KASTEN/2 - RADIUS + YOFF;
            x_alt = x;
            y_alt = y;
        }

        public void zeichnen(Graphics2D g)
        {
            g.setColor(farbe);

            x_alt = x;
            y_alt = y;

            g.fillOval(x,y,RADIUS*2,RADIUS*2);
            g.drawOval(x-RADIUS,y-RADIUS,RADIUS*4,RADIUS*4);
        }

        public void loeschen(Graphics2D g)
        {
          if(x_alt!=x || y_alt!=y)
          {
            g.setColor(Color.WHITE);

            g.fillOval(x_alt,y_alt,RADIUS*2,RADIUS*2);
            g.drawOval(x_alt-RADIUS,y_alt-RADIUS,RADIUS*4,RADIUS*4);
          }
        }

    }

    private class MausLauscher extends MouseAdapter
    {
        public void mouseEntered(MouseEvent e)
        {
            mauspos = kugelsim.getMousePosition();
            xneu = (int)mauspos.getX();
            yneu = (int)mauspos.getY();   
            xalt = xneu;
            yalt = yneu;

            zeit = 0;
            aktiviereTon();

            repaint();
        }
    }
}




